/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.realtime.json.protocol;

import java.util.List;
import com.openexchange.realtime.packet.ID;
import com.openexchange.realtime.packet.Stanza;
import com.openexchange.realtime.util.Duration;

/**
 * {@link RTClientState} - The {@link RTClientState} encapsulates the state of a connected client by keeping track of the sequenced and
 * unsequenced Stanzas that still have to be delivered to the client.
 *
 * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
 */
public interface RTClientState {

    /**
     * Called when the client sent an acknowledgement for a stanza
     * @param sequenceNumber
     */
    public abstract void acknowledgementReceived(long sequenceNumber);

    /**
     * Enqueues a stanza. If it contains a sequence number, it is enqueued in the resendBuffer, otherwise in the nonsequenceStanzas
     * @param stanza
     */
    public abstract void enqueue(Stanza stanza);

    /**
     * Retrieves a list of stanzas that are still to be transmitted to the client
     * @return
     */
    public abstract List<Stanza> getStanzasToSend();

    /**
     * A purge run removes all unsequenced stanzas from the state and increases the TTL counter of sequenced stanzas.
     */
    public abstract void purge();

    public abstract ID getId();

    public abstract void lock();

    public abstract void unlock();

    /**
     * Touch sets the last-seen timestamp for this state entry
     */
    public abstract void touch();

    /**
     * Retrieves the timestamp for when this user was last seen in milliseconds
     */
    public abstract long getLastSeen();

    /**
     * Checks whether this state should be considered timed out relative to the given timestamp
     * @param timestamp - The timestamp to check the timeout status for
     * @return true if the timestamp is more than thirty minutes ahead of the lastSeen timestamp, false otherwise
     */
    public abstract boolean isTimedOut(long timestamp);

    /**
     * Retrieve the duration of inactivity for this RTCLientState. Inactivity is defined as time the state wasn't actively touched by the
     * associated client. This combines calls to {@link RTClientState#getLastSeen()} and creating the nearest Duration via
     * {@link Duration#roundDownTo(long, java.util.concurrent.TimeUnit)}
     * 
     * @return the duration of inactivity for this RTCLientState
     */
    public abstract Duration getInactivityDuration();

    /**
     * Resets the state by clearing out sequenced and unsequenced stanzas. This is needed when a client wants to trigger a reset e.g. via
     * resetting the sequence number in use.
     */
    public abstract void reset();

}
